#ifndef _COMEDI_H
#define _COMEDI_H

/*
 * include file for data acquisition driver
 * David Schleef
 */

#define COMEDI_CONFIG 1
#define COMEDI_GETINFO 2
#define COMEDI_TRIG 3
#define COMEDI_GETPARAM 17
#define COMEDI_SETPARAM 18
#define COMEDI_GETNPARAM 19

#define COMEDI_TRIGNOW 0
#define COMEDI_TRIGCLK 1

#define COMEDI_RDONLY 1
#define COMEDI_RDWR 0
#define COMEDI_SHARED 2

#define COMEDI_IO 0
 #define COMEDI_IO_INPUT 0
 #define COMEDI_IO_OUTPUT 1
 #define COMEDI_IO_BIDIR 2
#define COMEDI_AD 1
 #define COMEDI_AD_ANALOG 1
 #define COMEDI_AD_DIGITAL 0
#define COMEDI_BITS 2
#define COMEDI_MAXDATA 3
#define COMEDI_RANGE 4 /* integer describing a range selection -- user
	must look up actual range in a library */
#define COMEDI_LASTDATA 5  /* last data sample */
#define COMEDI_LASTSEC 6  /* time of last data sample (seconds) */
#define COMEDI_LASTNSEC 6  /* time of last data sample (nanoseconds) */

#define COMEDI_DEGLITCH 32  /* true=1 */

#define COMEDI_STROBE 48  /* strobed output true=1 */
#define COMEDI_IOBITGROUP 49  /* how many bits are grouped together for
				i/o designation */
#define COMEDI_IOBITS 50  /* bitfield describing i/o of each signal line */
 #define COMEDI_IOBITS_INPUT 0
 #define COMEDI_IOBITS_OUTPUT 0xffffffff

#define COMEDI_COUNT 64  /* count number for counters & reset */


typedef struct{
	unsigned int chan;
	int io;
	unsigned int n;
	int trigsrc;
	int trigvar;
	int data;
}comedi_trig;

typedef struct{
	int chan;
	int data;
}comedi_sample;

typedef struct{
	char name[20];
	int iobase;
	int irq;
}comedi_devconfig;

typedef struct{
	unsigned int chan;
	unsigned int flags;
	unsigned int pnum;
	int pval;
}comedi_param;

typedef struct{
	int iobase;
	int iosize;
	int irq;
	int minor;
	int chan;
}comedi_info;

/*
typedef struct{
	int dest;
	int src;
	int nbytes;
}comedi_copy;
*/

#endif /* _COMEDI_H */

