/*
 * dummy driver
 * David A. Schleef
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/timer.h>
#include <asm/io.h>
#include <comedi_module.h>

/* this structure is for data unique to this hardware driver.  If
   several hardware drivers keep similar information in this structure,
   feel free to suggest moving the variable to the comedi_device struct.  */
typedef struct{
	int data;
}dummy_private;
#define devpriv ((dummy_private *)dev->private)

comedi_devinit dummy_init;
static int dummy_rem(comedi_device *dev);

static int dummy_input(comedi_device *dev,comedi_trig *it);
static int dummy_output(comedi_device *dev,comedi_trig *it);

int dummy_init(comedi_device *dev,comedi_devconfig *it)
{
/* there is a *_init() function for each hardware driver installed.
   Each one is called in sequence.
   if it->name is a device that this hardware driver supports, we try
   to initialize it.  If everything works, return a 1.  If there is
   an error, return the error.  If we do not match it->name, return a
   0 so that the next *_init() function can be tried. */

/* *dev is zeroed except for dev->minor, which contains the minor
   number being used.  *it has configuration information (not really
   used here--see other drivers.)  this function is responsible for
   filling the rest of *dev. */

	if(strcmp("dummy",it->name))
		return 0;

	printk("comedi%d: dummy: ",dev->minor);
	
	/* this is an appropriate place to put probe and initialization
	   code.  remember to fill in dev->iobase and friends. */

	dev->chan=2;

	dev->private=kmalloc(sizeof(dummy_private)+
		dev->chan*sizeof(comedi_chinfo),GFP_KERNEL);
	if(!dev->private)return -ENOMEM;
	memset(dev->private,0,sizeof(dummy_private)+
		dev->chan*sizeof(comedi_chinfo));

	dev->chinfo=dev->private+sizeof(dummy_private);

	dev->chinfo[0].trig=dummy_input;
	addparam(&dev->chinfo[0].paramlist,COMEDI_BITS,COMEDI_RDONLY,16);
	addparam(&dev->chinfo[0].paramlist,COMEDI_MAXDATA,COMEDI_RDONLY,0xffff);
	addparam(&dev->chinfo[0].paramlist,COMEDI_AD,COMEDI_RDONLY,
		COMEDI_AD_ANALOG);
	addparam(&dev->chinfo[0].paramlist,COMEDI_IO,COMEDI_RDONLY,
		COMEDI_IO_INPUT);

	dev->chinfo[1].trig=dummy_output;
	addparam(&dev->chinfo[1].paramlist,COMEDI_BITS,COMEDI_RDONLY,16);
	addparam(&dev->chinfo[1].paramlist,COMEDI_MAXDATA,COMEDI_RDONLY,0xffff);
	addparam(&dev->chinfo[1].paramlist,COMEDI_AD,COMEDI_RDONLY,
		COMEDI_AD_ANALOG);
	addparam(&dev->chinfo[1].paramlist,COMEDI_IO,COMEDI_RDONLY,
		COMEDI_IO_OUTPUT);

	comedi_initbuf(&dev->rdbuf);

	dev->rem=dummy_rem;

	return 1;
}


static int dummy_rem(comedi_device *dev)
{
	printk("comedi%d: dummy: remove\n",dev->minor);
	
	kfree(dev->private);

	comedi_freebuf(&dev->rdbuf);
	
	return 0;
}

static int dummy_input(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	if(it->trigsrc!=COMEDI_TRIGNOW)return -EINVAL;
	it->n=1;
	it->data=devpriv->data;
	s.chan=0;
	s.data=it->data;
	report_sample(dev,&s);

	return 0;
}

static int dummy_output(comedi_device *dev,comedi_trig *it)
{
	devpriv->data=it->data;

	if(it->trigsrc!=COMEDI_TRIGNOW)return -EINVAL;
	it->n=1;
	devpriv->data=it->data;
/*
	s.chan=0;
	s.data=it->data;
	report_sample(dev,*s);
*/

	return 0;
}

