
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <linux/malloc.h>
#include <comedi_module.h>
#include <ni_stc.h>

#define NI_SIZE 0x20

static char pnp_magic[]={
	0x00,0x00,0x6a,0xb5,0xda,0xed,0xf6,0xfb,
	0x7d,0xbe,0xdf,0x6f,0x37,0x1b,0x0d,0x86,
	0xc3,0x61,0xb0,0x58,0x2c,0x16,0x8b,0x45,
	0xa2,0xd1,0xe8,0x74,0x3a,0x9d,0xce,0xe7,
	0x73,0x39,0 };

#define PNP_CMD 0x479
#define PNP_ADR 0x279
#define PNP_RD 0x203

/* Additional windowed registers unique to E series */

#define Configuration_Memory_Clear	82
#define ADC_FIFO_Clear			83

/* i/o port offsets */

/* 8 bit registers */
#define Status				0x01
#define Serial_Command			0x0d
#define Misc_Command			0x0f
#define Port_A				0x19
#define Port_B				0x1b
#define Port_C				0x1d
#define Configuration			0x1f
#define Strobes				0x01
#define Channel_A_Mode			0x03
#define Channel_B_Mode			0x05
#define Channel_C_Mode			0x07
#define AI_AO_Select			0x09
#define G0_G1_Select			0x0b

/* 16 bit registers */
#define Configuration_Memory_Low	0x10
#define Configuration_Memory_High	0x12
#define ADC_FIFO_Data_Register		0x1c
#define AO_Configuration		0x16
#define DAC_FIFO_Data			0x1e
#define DAC0_Direct_Data		0x18
#define DAC1_Diredt_Data		0x1a


typedef struct{
	int dio;
}ni_private;
#define devpriv ((ni_private *)dev->private)

comedi_devinit ni_E_init;
static int ni_dio(comedi_device *dev,comedi_trig *ad);
static int ni_ad(comedi_device *dev,comedi_trig *it);
static int ni_rem(comedi_device *dev);
static int pnp_init(int iobase);

static int ni_ad(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;
	int gain;
	int i,to;
	
	switch(it->trigsrc){
	case COMEDI_TRIGNOW:
		win_out(1,ADC_FIFO_Clear);
		win_out(1,Configuration_Memory_Clear);

		gain=getparamval(dev->chinfo[it->chan].paramlist,COMEDI_RANGE);
		outw(0x1000|it->chan,dev->iobase+Configuration_Memory_High);
		outw(0x8000|((gain&0x8)<<5)|(gain&0x7),
			dev->iobase+Configuration_Memory_Low);
		win_out(1,AI_Command_1_Register);
		for(i=0;i<1000;i++){
			if(!(inw(AI_Status_1+dev->iobase)&
			  AI_FIFO_Empty_St)){
				win_out(1,ADC_FIFO_Clear);
				break;
			}
		}
	
		win_out(1,AI_Command_1_Register);
		
		for(i=0;i<1000;i++){
			if(!(inw(AI_Status_1+dev->iobase)&
			  AI_FIFO_Empty_St)){
				s.data=inw(dev->iobase+ADC_FIFO_Data_Register);
				if(!(gain&0x8))s.data^=0x800;
				s.data&=0xfff;
				s.chan=it->chan;
				it->data=s.data;
				report_sample(dev,&s);
				return 0;
			}
		}
		return -ETIME;
	case COMEDI_TRIGCLK:
		win_out(1,ADC_FIFO_Clear);
		win_out(1,Configuration_Memory_Clear);

		gain=getparamval(dev->chinfo[it->chan].paramlist,COMEDI_RANGE);
		outw(0x1000|it->chan,dev->iobase+Configuration_Memory_High);
		outw(0x8000|((gain&0x8)<<5)|(gain&0x7),
			dev->iobase+Configuration_Memory_Low);
		
		win_out(AI_Configuration_Start,Joint_Reset_Register);
		win_out(it->n>>8,AI_SC_Load_A_Registers);
		win_out(it->n&0xffff,AI_SC_Load_A_Registers+1);
		win_out(0x20,AI_Command_1_Register);
		/*win_out(0x60,AI_START_STOP_Select_Register);*/
		win_out(0,AI_SI_Load_A_Registers);
		win_out(1,AI_SI_Load_A_Registers+1);
		win_out(0x200,AI_Command_1_Register);
		win_out(0,AI_SI_Load_A_Registers);
		win_out(0x4e1f,AI_SI_Load_A_Registers+1);
		win_out(0x7cf,AI_SI2_Load_A_Register);
		win_out(0x7cf,AI_SI2_Load_B_Register);
		win_out(0x0100,AI_Mode_2_Register);
		win_out(0x0800,AI_Command_1_Register);
		win_out(0x0300,AI_Mode_2_Register);
		win_out(AI_Configuration_End,Joint_Reset_Register);

		win_out(1,ADC_FIFO_Clear);
		
		win_out(0x1540,AI_Command_1_Register);
		win_out(1,AI_Command_2_Register);

		to=0;
		for(i=0;i<it->n;i++){
			while(inw(AI_Status_1+dev->iobase)&AI_FIFO_Empty_St){
				if(to++>10000)return -ETIME;
				udelay(100);
			}
			s.data=inw(dev->iobase+ADC_FIFO_Data_Register);
			if(!(gain&0x8))s.data^=0x800;
			s.data&=0xfff;
			s.chan=it->chan;
			report_sample(dev,&s);
		}
		return 0;
	default:
		return -EINVAL;
	}
}

static int ni_adsp(comedi_device *dev,comedi_param *it)
{
	switch(it->pnum){
	case COMEDI_RANGE:
		if(it->pval<0 || it->pval>=16)return -EINVAL;
		changeparam(&dev->chinfo[it->chan].paramlist,COMEDI_RANGE,
			0,it->pval);
		return 0;
	default:
		return -EINVAL;
	}
}

static int ni_dio(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	switch(it->io){
	case COMEDI_IO_INPUT:
		s.data=it->data=
			win_in(DIO_Input_Register)&0xff;
		s.chan=it->chan;
	
		report_sample(dev,&s);
	
		return 0;
	case COMEDI_IO_OUTPUT:
		win_out(it->data&0xff,DIO_Output_Register);
		return 0;
	default:
		return -EINVAL;
	}
}

static int ni_do(comedi_device *dev,comedi_trig *it)
{
	return 0;
}

static int ni_dsp(comedi_device *dev,comedi_param *it)
{
	switch(it->pnum){
	case COMEDI_IOBITS:
		if(it->pval&~0xff)return -EINVAL;
		devpriv->dio=it->pval;
		win_out(devpriv->dio,DIO_Control_Register);
		changeparam(&dev->chinfo[it->chan].paramlist,COMEDI_IOBITS,0,
			it->pval);
		return 0;
	default:
		return -EINVAL;
	}
}

int ni_E_init(comedi_device *dev,comedi_devconfig *it)
{
	int i;

	if(strcmp("ni_E",it->name))
		return 0;

	printk("comedi%d: ni_E: 0x%04x\n",dev->minor,it->iobase);
	if(check_region(it->iobase,NI_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	dev->iobase=it->iobase;
	dev->iosize=NI_SIZE;
	dev->irq=0;

	dev->chan=19;

	devpriv=kmalloc(sizeof(ni_private)+dev->chan*sizeof(comedi_chinfo),
		GFP_KERNEL);
	if(!devpriv){
		return -ENOMEM;
	}
	memset(devpriv,0,sizeof(ni_private)+dev->chan*sizeof(comedi_chinfo));
	request_region(it->iobase,NI_SIZE,"ni_E");

	dev->chinfo=(void *)(devpriv+1);
	for(i=0;i<16;i++){
		dev->chinfo[i].trig=ni_ad;
		dev->chinfo[i].sp=NULL;
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_BITS,COMEDI_RDONLY,12);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_MAXDATA,COMEDI_RDONLY,0xfff);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_ANALOG);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_INPUT);
		addparam(&dev->chinfo[i].paramlist,COMEDI_RANGE,0,1);
	}

	dev->chinfo[18].trig=ni_dio;
	dev->chinfo[18].sp=ni_dsp;
	addparam(&dev->chinfo[18].paramlist,
		COMEDI_BITS,COMEDI_RDONLY,8);
	addparam(&dev->chinfo[18].paramlist,
		COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
	addparam(&dev->chinfo[18].paramlist,
		COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_BIDIR);
	addparam(&dev->chinfo[18].paramlist,
		COMEDI_IOBITGROUP,COMEDI_RDONLY,1);
	addparam(&dev->chinfo[18].paramlist,
		COMEDI_IOBITS,COMEDI_RDWR,0xff&COMEDI_IOBITS_INPUT);
	devpriv->dio=0;
	win_out(devpriv->dio,DIO_Control_Register);

	win_out(0x1b80,Clock_and_FOUT_Register); /* set timebase */
	win_out(AI_Configuration_Start,Joint_Reset_Register);
	win_out(0x3f80,Interrupt_A_Ack_Register); /* clear interrupts */
	win_out(0x0000,AI_Command_1_Register); /* don't do anything */
	win_out(0xa4a0,AI_Personal_Register); /* ? */
		/* enable STC output signals */
	win_out(0x032e,AI_Output_Control_Register);
		/* listen to start/stop */
	win_out(0x000d,AI_Mode_1_Register);
	win_out(0,AI_Mode_2_Register); /* ? */
	win_out(0,AI_Mode_3_Register); /* ? */
		/* start1 on AI_START1_Pulse */
	win_out(0x0060,AI_Trigger_Select_Register); /* trigger source */
		/* start on SI_TC */
		/* stop on AI_STOP_IN pin */
	win_out(0x29e0,AI_START_STOP_Select_Register);
	win_out(AI_Configuration_End,Joint_Reset_Register);
#if 0
	win_out(1,ADC_FIFO_Clear);
	win_out(AI_Reset,Joint_Reset_Register);  /* remove? */
	win_out(1,Configuration_Memory_Clear);  /* remove? */
	win_out(1,ADC_FIFO_Clear);  /* remove? */
	win_out(0x1b00,Clock_and_FOUT_Register); /* set timebase */
	win_out(AI_Configuration_Start,Joint_Reset_Register);
	win_out(0x3f80,Interrupt_A_Ack_Register); /* clear interrupts */
	win_out(0x0000,AI_Command_1_Register); /* don't do anything */
	win_out(0x000c,AI_Mode_1_Register); /* listen to start/stop cmds */
	win_out(AI_Configuration_End,Joint_Reset_Register);  /* remove? */
	win_out(AI_Configuration_Start,Joint_Reset_Register);  /* remove? */
	win_out(0x1b80,Clock_and_FOUT_Register); /* move? */
	win_out(0xa4a0,AI_Personal_Register); /* ? */
	win_out(0x032e,AI_Output_Control_Register); /* output signals */
	win_out(0x032e,AI_Output_Control_Register); /* remove? */
	win_out(AI_Configuration_End,Joint_Reset_Register);  /* remove? */
	win_out(AI_Configuration_Start,Joint_Reset_Register);  /* remove? */
	win_out(0,AI_Mode_2_Register); /* ? */
	win_out(0x032e,AI_Output_Control_Register); /* remove? */
	win_out(AI_Configuration_End,Joint_Reset_Register);  /* remove? */
	win_out(0,AI_Mode_3_Register); /* ? */
	win_out(AI_Configuration_Start,Joint_Reset_Register);  /* remove? */
	win_out(0x000d,AI_Mode_1_Register); /* move? */
	win_out(0x0060,AI_Trigger_Select_Register); /* trigger source */
	win_out(AI_Configuration_End,Joint_Reset_Register);  /* remove? */
	win_out(AI_Configuration_Start,Joint_Reset_Register);  /* remove? */
	win_out(0x0060,AI_START_STOP_Select_Register); /* remove? */
	win_out(AI_Configuration_End,Joint_Reset_Register);  /* remove? */
	win_out(AI_Configuration_Start,Joint_Reset_Register);  /* remove? */
		/* start on AI_START1_Pulse */
		/* stop on AI_STOP_IN pin */
	win_out(0x29e0,AI_START_STOP_Select_Register);
	win_out(AI_Configuration_End,Joint_Reset_Register);
	win_out(1,ADC_FIFO_Clear);  /* remove? */
#endif

	comedi_initbuf(&dev->rdbuf);

	dev->rem=ni_rem;

	return 1;
}

/* this is not used, since it doesn't seem to work.  use isapnptools
instead. */
static int pnp_init(int iobase)
{
	int i;

	/* initiate board */
	for(i=0;i<34;i++)
		outb(pnp_magic[i],PNP_ADR);

	/* reset pnp registers */
	outb(0x02,PNP_ADR);
	outb(0x07,PNP_CMD);
	
	/* initiate board */
	for(i=0;i<34;i++)
		outb(pnp_magic[i],PNP_ADR);

	/* set pnp isolation state */
	outb(0x03,PNP_ADR);
	outb(0x00,PNP_CMD);
	
	/* set pnp configuration state */
	outb(0x06,PNP_ADR);
	outb(0x01,PNP_CMD);
	
	/* assign pnp base address */
	
	outb(0x60,PNP_ADR);
	outb((iobase>>8)&0xff,PNP_CMD);
	outb(0x61,PNP_ADR);
	outb(iobase&0xff,PNP_CMD);
	
	/* assign pnp data port address */

	outb(0x00,PNP_ADR);
	outb(PNP_RD>>2,PNP_CMD);
	
	/* verify base address */
	
	outb(0x60,PNP_ADR);
	i=inb(PNP_RD)<<8;
	outb(0x61,PNP_ADR);
	i|=inb(PNP_RD);
	
printk("i=0x%x ",i);
	if(i!=iobase)return -EIO;
	
	/* activate board */

	outb(0x03,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x07,PNP_ADR);
	outb(0x00,PNP_CMD);

	outb(0x30,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x07,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x30,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x02,PNP_ADR);
	outb(0x02,PNP_CMD);
	
	return 0;
}


static int ni_rem(comedi_device *dev)
{
	int i;

	printk("comedi%d: ni_E: remove\n",dev->minor);
	
	for(i=0;i<dev->chan;i++)
		delparamlist(dev->chinfo[i].paramlist);
	kfree(devpriv);
	release_region(dev->iobase,dev->iosize);

	comedi_freebuf(&dev->rdbuf);

	return 0;
}

